<?php

class BasePayment extends WC_Payment_Gateway_CC
{
    public $name;
    public $instructions;
    public $apiHost;
    public $apiKey;
    public $orderPrefix;
    public $apiUrl;
    public $successName;
    public function __construct()
    {
        $this->init_form_fields();
        $this->init_settings();

        $this->name = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->instructions = $this->get_option('instructions', $this->description);
        $this->apiHost = $this->get_option('apiHost');
        $this->apiKey = $this->get_option('apiKey');
        $orderPrefix = '';
        if (isset($_SERVER['HTTP_HOST'])) {
            $orderPrefix = sanitize($_SERVER['HTTP_HOST']);
        }

        $this->orderPrefix = $orderPrefix . '_';

        //Save hook for settings
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
        add_filter('woocommerce_payment_complete_order_status', array($this, 'change_payment_complete_order_status'), 10, 3);
    }

    public function process_payment($order_id)
    {
        $order = wc_get_order($order_id);

        if ($order->get_total() > 0) {
            return $this->main_payment($order);
        }
    }

    public function base_success()
    {
        $soopay_order_id = sanitize($_GET['soopayOrderId'] ?? '');
        $order_id = sanitize($_GET['order_id'] ?? '');
        $order = wc_get_order($order_id);
        if ($order) {
            if ($order->get_status() == 'pending') {
                $api = $this->apiHost . '/v1/order?orderId=' . $soopay_order_id;
                $result = wp_post_json($api, wp_json_encode(array('key' => $this->apiKey)));
                $result = json_decode($result, true);
                $order_status = @$result['order']['status'];
                if ($order_status == 'completed') {
                    // $order->payment_complete();
                    $order->update_status('processing');
                    WC()->cart->empty_cart();
                    wc_redirect($this->get_return_url($order));
                } else {
                    if ($result['order']['statusNote']) {
                        wc_redirect($order->get_cancel_order_url() . '&order_note=' . $result['order']['statusNote']);
                    } else {
                        wc_redirect($order->get_cancel_order_url());
                    }
                }
            } else if ($order->get_status() == 'processing') {
                WC()->cart->empty_cart();
                wc_redirect($this->get_return_url($order));
            } else {
                wc_redirect($order->get_cancel_order_url());
            }
        }
        die();
    }

    /*
    $order
    {
      "id": 42,
      "parent_id": 0,
      "status": "pending",
      "currency": "USD",
      "version": "7.1.1",
      "prices_include_tax": false,
      "date_created": {
        "date": "2022-12-13 15:17:43.000000",
        "timezone_type": 1,
        "timezone": "+00:00"
      },
      "date_modified": {
        "date": "2022-12-13 15:17:43.000000",
        "timezone_type": 1,
        "timezone": "+00:00"
      },
      "discount_total": "0",
      "discount_tax": "0",
      "shipping_total": "0",
      "shipping_tax": "0",
      "cart_tax": "0",
      "total": "0.99",
      "total_tax": "0",
      "customer_id": 1,
      "order_key": "wc_order_xUkHTzRb0HwhW",
      "billing": {
        "first_name": "",
        "last_name": "",
        "company": "",
        "address_1": "",
        "address_2": "",
        "city": "",
        "state": "",
        "postcode": "",
        "country": "",
        "email": "",
        "phone": ""
      },
      "shipping": {
        "first_name": "",
        "last_name": "",
        "company": "",
        "address_1": "",
        "address_2": "",
        "city": "",
        "state": "",
        "postcode": "",
        "country": "",
        "phone": ""
      },
      "payment_method": "soopay-creditcard",
      "payment_method_title": "Soopay CreditCard Plugin for Woocommerce",
      "transaction_id": "",
      "customer_ip_address": "113.128.75.216",
      "customer_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36",
      "created_via": "checkout",
      "customer_note": "",
      "date_completed": null,
      "date_paid": null,
      "cart_hash": "9ed21a8eb87503d75b519d3ffce97e6a",
      "order_stock_reduced": false,
      "download_permissions_granted": false,
      "new_order_email_sent": false,
      "recorded_sales": false,
      "recorded_coupon_usage_counts": false,
      "number": "42",
      "meta_data": [
        {
          "id": 283,
          "key": "is_vat_exempt",
          "value": "no"
        }
      ],
      "line_items": {
        "10": {}
      },
      "tax_lines": [],
      "shipping_lines": [],
      "fee_lines": [],
      "coupon_lines": []
    }
     */
    public function main_payment($order)
    {
        $total = intval($order->get_total());
        $param = json_decode(file_get_contents("php://input"), true);

        $jsonForm = array();
        $jsonForm['key'] = $this->apiKey;
        $jsonForm['merchantOrderId'] = $this->orderPrefix . $order->get_id();
        $jsonForm['amount'] = $order->get_total();
        $jsonForm['currency'] = $order->get_currency();
        $successUrl = WC()->api_request_url($this->successName);
        $jsonForm['successUrl'] = strpos($successUrl, '?') === false ? $successUrl . '?order_id=' . $order->get_id() : $successUrl . '&order_id=' . $order->get_id();
        $jsonForm['failUrl'] = $order->get_cancel_order_url();
        // $jsonForm['ip'] = $order->get_customer_ip_address();
        $jsonForm['ip'] = $this->get_ip_address();
        $jsonForm['email'] = $order->get_billing_email();
        $jsonForm['phone'] = $order->get_billing_phone();
        $jsonForm['billingFirstName'] = $order->get_billing_first_name();
        $jsonForm['billingLastName'] = $order->get_billing_last_name();
        $jsonForm['billingCountry'] = $order->get_billing_country();
        $jsonForm['billingState'] = $order->get_billing_state();
        $jsonForm['billingCity'] = $order->get_billing_city();
        $jsonForm['billingState'] = $order->get_billing_state();
        $jsonForm['billingLine1'] = $order->get_billing_address_1();
        $jsonForm['billingLine2'] = $order->get_billing_address_2();
        $jsonForm['billingPostCode'] = $order->get_billing_postcode();
        $jsonForm['plugin'] = 'wordpress_v1.5.6';

        $jsonForm = $this->form_extra_field($jsonForm);
        $url = $this->apiHost . $this->apiUrl;
        $args = array(
            'body' => http_build_query($jsonForm),
            'timeout' => 30,
            'sslverify' => false,
        );

        $response = wp_remote_post($url, $args);
        $response_body = wp_remote_retrieve_body($response);
        $createPayResponse = json_decode($response_body);

        if ($createPayResponse->{'status'} == 'success') {
            // $order->payment_complete();
            $order->update_status('processing');
            WC()->cart->empty_cart();
            return [
                'result' => 'success',
                'redirect' => $createPayResponse->{'redirectUrl'},
            ];
        } else if ($createPayResponse->{'status'} == '3ds') {
            return [
                'result' => 'success',
                'redirect' => $createPayResponse->{'redirectUrl'},
            ];
        } else {
            wc_add_notice($createPayResponse->{'statusNote'}, 'error');
            return;
        }
    }

    public function form_extra_field($data)
    {
        return $data;
    }

    public function change_payment_complete_order_status($status, $order_id = 0, $order = false)
    {
        if ($order && $this->id === $order->get_payment_method()) {
            $status = 'completed';
        }
        return $status;
    }

    public static function get_ip_address()
    {
        if (isset($_SERVER['HTTP_X_CLIENT_IP'])) {
            return sanitize($_SERVER['HTTP_X_CLIENT_IP']);
        } elseif (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            // Proxy servers can send through this header like this: X-Forwarded-For: client1, proxy1, proxy2
            // Make sure we always only send through the first IP in the list which should always be the client IP.
            return trim(current(preg_split('/,/', sanitize($_SERVER['HTTP_X_FORWARDED_FOR']))));
        } elseif (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) {
            return sanitize($_SERVER['HTTP_CF_CONNECTING_IP']);
        } elseif (isset($_SERVER['HTTP_FASTLY_CLIENT_IP'])) {
            return sanitize($_SERVER['HTTP_FASTLY_CLIENT_IP']);
        } elseif (isset($_SERVER['HTTP_TRUE_CLIENT_IP'])) {
            return sanitize($_SERVER['HTTP_TRUE_CLIENT_IP']);
        } elseif (isset($_SERVER['HTTP_X_REAL_IP'])) {
            return sanitize($_SERVER['HTTP_X_REAL_IP']);
        } elseif (isset($_SERVER['HTTP_X_CLUSTER_CLIENT_IP'])) {
            return sanitize($_SERVER['HTTP_X_CLUSTER_CLIENT_IP']);
        } elseif (isset($_SERVER['HTTP_X_FORWARDED'])) {
            return sanitize($_SERVER['HTTP_X_FORWARDED']);
        } elseif (isset($_SERVER['HTTP_FORWARDED_FOR'])) {
            return sanitize($_SERVER['HTTP_FORWARDED_FOR']);
        } elseif (isset($_SERVER['HTTP_FORWARDED'])) {
            return sanitize($_SERVER['HTTP_FORWARDED']);
        } elseif (isset($_SERVER['HTTP_APPENGINE_USER_IP'])) {
            return sanitize($_SERVER['HTTP_APPENGINE_USER_IP']);
        } elseif (isset($_SERVER['REMOTE_ADDR'])) {
            return sanitize($_SERVER['REMOTE_ADDR']);
        } elseif (isset($_SERVER['HTTP_CF_PSEUDO_IPV4'])) {
            return sanitize($_SERVER['HTTP_CF_PSEUDO_IPV4']);
        }

        return '';
    }
}
