<?php

use Inc\Base\StripeCheckoutDescription;

class WCCreditCardDirectPayment extends BasePayment
{
    public function __construct()
    {
        $this->id = 'soopay-creditcard';
        $this->has_fields = false;
        $this->icon = plugins_url('../assets/img/creditcard.svg', dirname(dirname(__FILE__)));
        $this->method_title = __('Soopay', 'soopay-for-woocommerce');
        $this->method_description = __('Soopay CreditCard Direct Plugin for Woocommerce.', 'soopay-for-woocommerce');
        $this->title = __('Credit Card', 'soopay-for-woocommerce');
        $this->apiUrl = '/v1/direct/pay';
        $this->supports[] = 'default_credit_card_form';
        $stripeCheckoutDescription = new StripeCheckoutDescription();
        $stripeCheckoutDescription->generateDescriptionFields();
        $this->successName='creditcard_success';
        add_action('woocommerce_api_creditcard_success', array($this, $this->successName));
        parent::__construct();
    }

    public function init_form_fields()
    {
        $this->form_fields = [
            'apiHost' => array(
                'title' => __('API host', 'soopay-for-woocommerce'),
                'type' => 'text',
                'default' => __('https://api.soopay.io', 'soopay-for-woocommerce'),
                'desc_tip' => true,
                'description' => __('Soopay host', 'soopay-for-woocommerce')
            ),

            'apiKey' => array(
                'title' => __('API key', 'soopay-for-woocommerce'),
                'type' => 'text',
                'default' => __('098f6bcd4621d373cade4e832627b4f6', 'soopay-for-woocommerce'),
                'desc_tip' => true,
                'description' => __('Soopay key', 'soopay-for-woocommerce')
            ),
        ];
    }
    
    public function creditcard_success()
    {
        $this->base_success();
    }

    public function form()
    {
        wp_enqueue_script('wc-credit-card-form');

        $fields = array();

        $default_fields = array(
            'card-number-field' => '<p class="form-row form-row-wide">
                <label for="' . esc_attr($this->id) . '-card-number">' . esc_html__('Card number', 'soopay-for-woocommerce') . '&nbsp;<span class="required">*</span></label>
                <input id="' . esc_attr($this->id) . '-card-number" class="input-text wc-credit-card-form-card-number" inputmode="numeric" autocomplete="cc-number" autocorrect="no" autocapitalize="no" spellcheck="no" type="tel" placeholder="&bull;&bull;&bull;&bull; &bull;&bull;&bull;&bull; &bull;&bull;&bull;&bull; &bull;&bull;&bull;&bull;" ' . $this->field_name('card-number') . ' />
              </p>',
            'card-expiry-field' => '<p class="form-row form-row-first">
                <label for="' . esc_attr($this->id) . '-card-expiry">' . esc_html__('Expiry', 'soopay-for-woocommerce') . '&nbsp;<span class="required">*</span></label>
                <input id="' . esc_attr($this->id) . '-card-expiry" class="input-text wc-credit-card-form-card-expiry" inputmode="numeric" autocomplete="off" autocorrect="no" autocapitalize="no" spellcheck="no" type="tel" maxlength="7" placeholder="' . esc_attr__('MM / YY', 'soopay-for-woocommerce') . '" ' . $this->field_name('card-expiry') . ' />
              </p>',
            'card-cvc-field' => '<p class="form-row form-row-last">
                <label for="' . esc_attr($this->id) . '-card-cvc">' . esc_html__('CVC', 'soopay-for-woocommerce') . '&nbsp;<span class="required">*</span></label>
                <input id="' . esc_attr($this->id) . '-card-cvc" class="input-text wc-credit-card-form-card-cvc" inputmode="numeric" autocomplete="off" autocorrect="no" autocapitalize="no" spellcheck="no" type="tel" maxlength="4" placeholder="' . esc_attr__('CVC', 'soopay-for-woocommerce') . '" ' . $this->field_name('card-cvc') . ' />
              </p>'
        );

        $fields = wp_parse_args($fields, apply_filters('woocommerce_credit_card_form_fields', $default_fields, $this->id));
        ?>

        <fieldset id="wc-<?php echo esc_attr($this->id); ?>-cc-form" class='wc-credit-card-form wc-payment-form'>
            <?php do_action('woocommerce_credit_card_form_start', $this->id); ?>
            <?php
            foreach ($fields as $field) {
                echo $field; // phpcs:ignore WordPress.XSS.EscapeOutput.OutputNotEscaped
            }
            ?>
            <?php do_action('woocommerce_credit_card_form_end', $this->id); ?>
        </fieldset>
        <?php
    }

    public function form_extra_field($data)
    {
        $cardVal = WC()->checkout()->get_value('soopay-creditcard-card-number');
        $expiryYear = substr(WC()->checkout()->get_value('soopay-creditcard-card-expiry'), 5, 4);
        $expiryMonth = (int) substr(WC()->checkout()->get_value('soopay-creditcard-card-expiry'), 0, 2);
        $cvc = WC()->checkout()->get_value('soopay-creditcard-card-cvc');

        $data['cardNumber'] = $cardVal;
        $data['cardExpYear'] = $expiryYear;
        $data['cardExpMonth'] = $expiryMonth;
        $data['cardCvc'] = $cvc;
        return $data;
    }
}
