<?php
/**
 * @package SoopayForWoocommerce
 */

/*
 * Plugin Name: Soopay
 * Description: Soopay Plugin for Woocommerce.
 * Author: Soopay
 * Version: 1.5.6
 * License: GPLv2 or later
 * Text Domain: soopay-for-woocommerce
 * Requires PHP: 7.4
 * Requires Plugins: woocommerce
 * Requires at least: 6.5
 * WC requires at least: 9.2
 * WC tested up to: 9.7
 */

use YahnisElsts\PluginUpdateChecker\v5\PucFactory;
use Automattic\WooCommerce\Utilities\FeaturesUtil;

defined('ABSPATH') or die('Not Allowed');

add_action('before_woocommerce_init', function() {
    // HPOS
    if (class_exists(FeaturesUtil::class)) {
        FeaturesUtil::declare_compatibility(
            'custom_order_tables',
            __FILE__,
            true
        );
    }

    // Blocks
    FeaturesUtil::declare_compatibility(
        'cart_checkout_blocks',
        __FILE__,
        true
    );
});

if (file_exists(dirname(__FILE__) . '/vendor/autoload.php')) {
    require_once dirname(__FILE__) . '/vendor/autoload.php';
}

if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    return;
}

$soopay_plugin_url = plugin_dir_url(__FILE__);
define('SOOPAY_PLUGIN_URL',$soopay_plugin_url);

require 'vendor/plugin-update-checker/plugin-update-checker.php';
require_once dirname(__FILE__) . '/inc/function.php';

$myUpdateChecker = PucFactory::buildUpdateChecker(
    'https://soopay.io/v1/soopay-for-woocommerce.json',
    __FILE__, //Full path to the main plugin file or functions.php.
    'soopay'
);

add_action('send_headers', function() {
    header("Referrer-Policy: same-origin");
});

add_action('plugins_loaded', 'soopay_init',0);
function soopay_init()
{
    if (!class_exists('WC_Payment_Gateway_CC')) {
        return;
    }
    $dir = dirname(__FILE__) . '/inc/payment-methods/classic/';
    $files = scandir($dir);
    foreach ($files as $file) {
        if ($file === '.' || $file === '..') {
            continue;
        }
        $filePath = $dir . $file;
        if (is_file($filePath)) {
            require_once $filePath;
        }
    }

    add_filter('woocommerce_payment_gateways', 'add_soopay_payment_gateway');
    function add_soopay_payment_gateway($gateways)
    {
        if (!class_exists('WC_Payment_Gateway_CC')) {
            return $gateways;
        }
         $class = [
          'WCCreditCardCheckoutPayment',
          'WCCreditCardDirectPayment',
          'WCPayPalPayment',
          'WCCryptoCurrencyCheckoutPayment'
        ];
        
        $gateways = array_merge($gateways, $class);
    
        return $gateways;
    }
    // require_once( WOO_GPPDIR . 'class-wc-gateway-ms-custom.php' );

}

add_action('template_redirect', function () {
    $order_note = sanitize($_GET['order_note']??'');
    wc_add_notice($order_note, 'error');
});

add_action('wp_enqueue_scripts', function() {
    if (is_checkout()) {
        wp_enqueue_style(
            'soopay_plugin_stylesheet',
            plugins_url('./assets/css/mainStyle.css', __FILE__),
            [],
            filemtime(plugin_dir_path(__FILE__) . 'assets/css/mainStyle.css')
        );
    }
});

/**
 * Custom function to register a payment method type
 */
function soopay_block_init()
{
    // Check if the required class exists
    if (!class_exists('Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType')) {
        return;
    }

    $dir = dirname(__FILE__) . '/inc/payment-methods/block/';
    $files = scandir($dir);

    foreach ($files as $file) {
        if ($file === '.' || $file === '..') {
            continue;
        }
        $filePath = $dir . $file;
      
        if (is_file($filePath)) {
         
            require_once $filePath;
        }
    }

    // Hook the registration function to the 'woocommerce_blocks_payment_method_type_registration' action
    add_action(
        'woocommerce_blocks_payment_method_type_registration',
        function (Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry) {
            $payment_method_registry->register(new WCCreditCardCheckoutPaymentBlock);
            $payment_method_registry->register(new WCPayPalPaymentBlock);
            $payment_method_registry->register(new WCCryptoCurrencyCheckoutPaymentBlock);
        }
    );
}
add_action('woocommerce_blocks_loaded', 'soopay_block_init');
